/**
 * 
 * sensed - version 0.2
 * 
 * A program that periodically polls a SCSI device (using the
 * SCSI REQUEST SENSE command) and launches commands depending
 * on the response.
 * 
 * It is meant to be used in conjunction with scanners that have
 * a "scan" button and report its status via the SENSE response.
 *
 * Since it makes use of the SANE SCSI core, it has to be linked 
 * against libsanei.a as if it was a SANE backend. Also requires
 * sane_strstatus.o. You'll find these lying around somewhere if
 * you have built SANE on your machine :-)
 * 
 * This version is a very crude "first attempt". It is not 
 * configurable except by changing the #define lines below, 
 * and in the header file. The header file determines how the
 * REQUEST SENSE response is interpreted, and it might have to
 * be changed depending on the scanner make and model.
 * This code was initialliy designed for (and works with) the
 * Fujitsu M3091DCd.
 *
 * Future versions should support different scanners and be
 * configurable using a config file or at least the command
 * line, but this is better than nothing.
 * 
 * This is Free Software. Use at your own risk.
 * Written in March 2001 by Frederik Ramm <frederik@remote.org>
 *
 * Changelog:
 *   v0.2 - release device during scan.
 * 
 */
#include "sane/config.h"

#include <errno.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include <sys/types.h>
#include <unistd.h>

#include "sane/sanei_scsi.h"

#include "sensed.h"

/* ------------------------------------------------------------
 * This is where you configure what to do if either the "scan"
 * or the "send" button are pressed. The "%d"s in the command
 * will be replaced by 
 *  - the duplex button status (1 or 0)
 *  - the number of copies wheel value (0 to 10)
 *  - the density wheel value (1 to 5)
 * in that order. You'll typically want to call a perl or shell
 * script that runs a command-line frontend like "scanimage" or
 * "scanadf" (http://www.martoneconsulting.com/sane-scanadf.html)
 * and then does something meaningful with the resulting images.
 */
#define SEND_COMMAND "/usr/local/bin/launch-scan send %d %d %d"
#define SCAN_COMMAND "/usr/local/bin/launch-scan scan %d %d %d"
/*
 * INTERVAL specifies how many milliseconds to wait between the
 * commands. The scanner I originally wrote this for will buffer
 * the button status for three seconds, i.e. it's ok to ask 
 * every two seconds.
 * DEVICE is the device to which the scanner is attached.
 */
#define INTERVAL     2000
#define DEVICE       "/dev/sga"

/* ------------------------------------------------------------ */


static void requestSense (int *fd, int debug)
{
  size_t responseLength = 255;
  unsigned char response[255];
  unsigned char command[1024];
  int ret;

  memset (response, '\0', sizeof (response));  

  setSenseAllocationLength (requestSenseCommand, responseLength);

  ret = sanei_scsi_cmd (*fd, requestSenseCommand, sizeof(requestSenseCommand), response, &responseLength);
 
  if ((getSenseASC(response) == 0x80) && (getSenseSKSV (response) == 0)) /* no sense */
    {
      if (debug)
        {
          printf ("Request Sense Results:\n");
          printf ("\tADF has paper: %s\n", getSenseADFHasPaper (response) ? "yes" : "no");
          printf ("\tPaper edge sensor has paper: %s\n", getSensePaperEdgeHasPaper (response) ? "yes" : "no");
          printf ("\tDuplex button: %spressed\n", getSenseDuplexButtonStatus (response) ? "" : "not ");
          printf ("\tScan button: %spressed\n", getSenseScanButtonStatus (response) ? "" : "not ");
          printf ("\tSend button: %spressed\n", getSenseSendButtonStatus (response) ? "" : "not ");
          printf ("\tDensity dial setting: %u\n", getSenseDensityDialSetting (response));
          printf ("\tNumber of copies set: %u\n", getSenseNumberOfCopies (response));
        }
      else
        {
          sanei_scsi_close (*fd);
          *fd = -1;

          /* we do this using "system" instead of fork/exec since we don't want to monitor the
           * scanner while the program is running anyway...
           */
          if (getSenseScanButtonStatus (response))
            {
              sprintf (command, SCAN_COMMAND, getSenseDuplexButtonStatus (response), 
                 getSenseNumberOfCopies (response), getSenseDensityDialSetting (response));
              system (command);
            }
          else if (getSenseSendButtonStatus (response))
            {
              sprintf (command, SEND_COMMAND, getSenseDuplexButtonStatus (response), 
                 getSenseNumberOfCopies (response), getSenseDensityDialSetting (response));
              system (command);
            }
        }
    }
  else
    {
      /* This is only interesting for debug mode because otherwise we don't care... if the scanner
       * is busy or jammed or whatever, then we don't initiate anything
       */
      if (debug)
        {
          printf ("Request sense ASC=%d, SKSV=%d, cannot decode\n", getSenseASC(response), getSenseSKSV (response));
        }
    }
}

int main (int argc, char *argv[])
{

  int fileHandle; 
  SANE_Status openRet;

  openRet = sanei_scsi_open (DEVICE, &fileHandle, 0, 0);
  if (openRet != SANE_STATUS_GOOD)
    {
      fprintf (stderr, "sensed: cannot open specified device '%s'\n", DEVICE);
      fprintf (stderr, "SANE says: %s\n", sane_strstatus (openRet));
      exit (1);
    }

  if (argc > 1)
    {
      if (!strcmp ("-d", argv[1]))
        {
          /* debug mode, only run once and print results */
          requestSense (&fileHandle, 1);
        }
      else
        {
          printf ("usage:\n%s [-d]\nWith -d (debug) switch, will query device once and print results.\n", argv[0]);
          printf ("Otherwise, will query device every %d microseconds and launch the following:\n", INTERVAL);
          printf ("for the 'Scan' button: %s\nfor the 'Send' button: %s\n", SCAN_COMMAND, SEND_COMMAND);
        }
     exit (0);
    }

  while (1)
    {
      requestSense (&fileHandle, 0);
      if (fileHandle == -1)
        {
  openRet = sanei_scsi_open (DEVICE, &fileHandle, 0, 0);
  if (openRet != SANE_STATUS_GOOD)
    {
      fprintf (stderr, "sensed: cannot open specified device '%s'\n", DEVICE);
      fprintf (stderr, "SANE says: %s\n", sane_strstatus (openRet));
      exit (1);
    }

        }
      usleep (INTERVAL*1000);
    }

  exit (0);

}
